@@ -2,23 +2,25 @@ package Method::Cached::KeyRule;
 
 use strict;
 use warnings;
-use Digest::SHA qw/sha1_base64/;
+use Digest::SHA;
 use JSON::XS;
-use Storable qw/freeze/;
-use Scalar::Util qw/refaddr/;
+use Storable;
+use Scalar::Util;
 
-sub regularize {
-    my $key_rule = shift;
+{
     no strict 'refs';
-    my $ref = ref $key_rule;
-    $ref || return &{$key_rule || 'LIST'}(@_);
-    $ref eq 'CODE' && return $key_rule->(@_);
-    my $key;
-    for my $rule (@{$key_rule}) {
-        $key = '';
-        $key = ref $rule ? $rule->(@_) : &{$rule}(@_);
+
+    sub regularize {
+        my $key_rule = shift;
+        my $ref = ref $key_rule;
+        $ref || return &{$key_rule || 'LIST'}(@_);
+        $ref eq 'CODE' && return $key_rule->(@_);
+        my $key;
+        for my $rule (@{$key_rule}) {
+            $key = ref $rule ? $rule->(@_) : &{$rule}(@_);
+        }
+        return $key;
     }
-    return $key;
 }
 
 sub SELF_SHIFT {
@@ -30,16 +32,16 @@ sub SELF_SHIFT {
 sub SELF_CODED {
     my ($method_name, $args) = @_;
     our $ENCODER ||= JSON::XS->new->convert_blessed(1);
-    *UNIVERSAL::TO_JSON = sub { freeze \@_ };
+    *UNIVERSAL::TO_JSON = sub { Storable::nfreeze \@_ };
     my $json = $ENCODER->encode($args->[0]);
     undef *UNIVERSAL::TO_JSON;
-    $args->[0] = sha1_base64($json);
+    $args->[0] = Digest::SHA::sha1_base64($json);
     return;
 }
 
 sub PER_OBJECT {
     my ($method_name, $args) = @_;
-    $args->[0] = refaddr $args->[0];
+    $args->[0] = Scalar::Util::refaddr $args->[0];
     return;
 }
 
@@ -49,14 +51,24 @@ sub LIST {
     $method_name . join chr(28), @{$args};
 }
 
+sub HASH {
+    my ($method_name, $args) = @_;
+    local $^W = 0;
+    my ($ser, %hash) = (q{}, @{$args});
+    map {
+        $ser .= chr(28) . $_ . (defined $hash{$_} ? '=' . $hash{$_} : q{})
+    } sort keys %hash;
+    $method_name . $ser;
+}
+
 sub SERIALIZE {
     my ($method_name, $args) = @_;
     local $^W = 0;
     our $ENCODER ||= JSON::XS->new->convert_blessed(1);
-    *UNIVERSAL::TO_JSON = sub { freeze \@_ };
+    *UNIVERSAL::TO_JSON = sub { Storable::nfreeze \@_ };
     my $json = $ENCODER->encode($args);
     undef *UNIVERSAL::TO_JSON;
-    $method_name . sha1_base64($json);
+    $method_name . Digest::SHA::sha1_base64($json);
 }
 
 1;
